class Screen
  
  @@project = nil

  def initialize
    @menu = []

    system "clear"
    info
    puts
    puts "-------"
    puts
    menu
    handle_menu
  end

  def menu
    @menu.each_with_index do |item, idx|
      puts "#{idx + 1}) #{item}"
    end
  end

  protected

  def show_projects show_id
    if Project.all.length == 0
      puts "No projects yet."
    else
      puts "## Your Projects\n\n"
      Project.all.each do |p|
        if show_id
          puts "- #{p.name} (#{p.id})"
        else
          puts "- #{p.name}"
        end
      end
    end
  end

  def show_project_heading
    puts "You are working on #{@@project.name}"
  end

end

class WelcomeScreen < Screen

  def info
    show_projects false
  end

  def menu
    if Project.all.length > 0
      @menu.push "New project"
      @menu.push "Choose project"
    else
      @menu.push "New project"
    end
    super
  end

  def handle_menu
    answer = gets.chomp

    if answer == "1"
      CreateProjectScreen.new
    elsif answer == "2"
      ChooseProjectScreen.new
    end
  end

end

class ChooseProjectScreen < Screen

  def info
    show_projects true
  end

  def menu
    puts "Type the number or name of the project"
  end

  def handle_menu
    answer = gets.chomp

    if answer.length == 1
      @@project = Project[:id => answer]
    else
      @@project = Project[:name => answer]
    end

    if @@project
      ShowProjectScreen.new
    else
      system "clear"
      puts "You are a bad typer, or maybe that project does not exist."
      sleep(2)
      ChooseProjectScreen.new
    end
  end

end

class CreateProjectScreen < Screen

  def info
    puts ""
  end

  def menu
    puts "Type the name of your new project"
  end

  def handle_menu
    name = gets.chomp
    
    system "clear"

    puts "Type the rate per hour for this project (ex: 60)"
    rate = gets.chomp

    @@project = Project.create(:name => name, :rate_per_hour => rate)
    ShowProjectScreen.new
  end

end

class ShowProjectScreen < Screen

  def info
    show_project_heading
  end

  def menu
    @menu.push "Start a work session"
    @menu.push "Show work completed"
    @menu.push "Create invoice"
    @menu.push "Go to welcome screen"
    @menu.push "Delete this project"
    super
  end

  def handle_menu
    answer = gets.chomp

    if answer == "1"
      DoWorkScreen.new
    elsif answer == "2"
      ShowSessionsForProjectScreen.new
    elsif answer == "3"
      CreateInvoiceScreen.new
    elsif answer == "4"
      WelcomeScreen.new
    elsif answer == "5"
      DeleteProjectScreen.new
    end
  end

end

class DoWorkScreen < Screen

  def initialize
    @session = Session.new
    super
  end

  def info
    show_project_heading
  end

  def menu
    @menu.push "Stop Working"
    @menu.push "Do not record this work."
    super
  end

  def handle_menu
    answer = gets.chomp

    if answer == "1"
      puts "Enter a description for the work you just completed."
      description = gets.chomp
      @session.stop({ :description => description, :project => @@project })
      system "clear"
      puts "You worked for #{seconds_to_hms(@session.elapsed_time)} on \"#{@session.description}\" and earned #{@session.earned}"
      sleep(4)

      ShowProjectScreen.new
    elsif answer == "2"
      # TODO put in check to verify user wants to discard work
      ShowProjectScreen.new
    end
  end

end

class ShowSessionsForProjectScreen < Screen

  def info
    show_project_heading

    puts

    if @@project.sessions.length > 0
      puts "## Work Completed\n"
      @@project.sessions.each do |s|
        puts "#{s.description} \t #{seconds_to_hms(s.elapsed_time)} \t #{s.earned}"
      end

      puts "\n## Total"
      puts "#{seconds_to_hms(@@project.total_time)} \t #{@@project.total_earned}"
    else
      puts "You haven't worked on this project yet."
    end
  end

  def menu
    @menu.push "Go back to project"
    super
  end

  def handle_menu
    answer = gets.chomp

    if answer == "1"
      ShowProjectScreen.new
    end
  end

end

class DeleteProjectScreen < Screen

  def info
    puts "Are you sure you want to delete #{@@project.name}"
  end

  def menu
    @menu.push "Yes"
    @menu.push "NO"
    super
  end

  def handle_menu
    answer = gets.chomp

    if answer == "1"
      @@project.delete
      WelcomeScreen.new
    else
      ShowProjectScreen.new
    end
  end

end

class CreateInvoiceScreen < Screen

  def info
    HTMLInvoiceExporter.new @@project
  end
  
  def menu
    @menu.push "Back to project"
    super
  end
  
  def handle_menu
    answer = gets.chomp
    ShowProjectScreen.new
  end

end

WelcomeScreen.new